import static org.junit.Assert.*;

import java.util.Observable;
import java.util.Observer;

import dtu.se1.MyStack;

import org.junit.Test;


public class TestStack {

	@Test
	public void testPushTop() {
		MyStack<Integer> stack = new MyStack<Integer>();
		int top = 10;
		stack.push(top);
		assertEquals(top,(int)stack.top());
	}

	@Test
	public void testPushPop() {
		MyStack<Integer> stack = new MyStack<Integer>();
		int top = 10;
		stack.push(top);
		assertEquals(top,(int)stack.pop());
		assertEquals(0,stack.size());
	}
	
	@Test
	public void testPushPopMore() {
		MyStack<Integer> stack = new MyStack<Integer>();
		stack.push(1);
		stack.push(2);
		stack.push(3);
		assertEquals(3,(int)stack.pop());
		assertEquals(2,(int)stack.pop());
		assertEquals(1,(int)stack.pop());
		assertEquals(0,stack.size());
	}
	
	@Test
	public void testToString() {
		MyStack<Integer> stack = new MyStack<Integer>();
		stack.push(1);
		stack.push(2);
		stack.push(3);
		assertEquals("Stack [ 1 2 3 ]",stack.toString());
	}
	
	@Test
	public void testObserver() {
		/* More elegantly, the test could be written using a mock object
		 * as the observer.
		 * 
		 * Note that methods of inner classes defined in a method cannot change 
		 * the value of temporary variables in that method. This means, the temporary
		 * variable has to be defined as final. However, the object that variable points to
		 * can be changed without problems. That is the reason for the inner class Counter, which
		 * otherwise would not be necessary. 
		 */
		
		final Counter countUpdateExecuted = new Counter();
		
		MyStack<Integer> stack = new MyStack<Integer>();
		
		Observer o = new Observer() { 
			public void update(Observable o, Object aspect) {
				countUpdateExecuted.inc();
			};
		};
		
		stack.addObserver(o);
		
		stack.push(3);
		stack.pop();
		
		assertEquals(2,countUpdateExecuted.getValue());
		
	}
	
	class Counter {
		private int value = 0;
		
		public void inc() { value++; }
		public int getValue() { return value; }
	}

}

